package client;

import general.*;

import java.io.IOException;
import java.util.Date;


public class Client extends User implements Runnable
{
	private String name;

	public Client(String server, int port, String name, ClientGUI userGUI)
	{
		super(server, port, userGUI);
		this.name = name;
	}

	@Override
	public boolean connect()
	{
		if (super.connect())
		{
			new Thread(this).start();
			try
			{
				outputStream.writeObject(name);
			} catch (IOException e)
			{
				String err = "Exception on logging in : " + e;
				userGUI.append(err);
				close();
				return false;
			}
		}
		return true;
	}

	@Override
	public void run()
	{
		while (true)
		{
			Packet packet;
			try
			{
				packet = (Packet) inputStream.readObject();
				if (new Date().getTime() - packet.getTimeStamp().getTime() >= 60 * 1000)
					continue;
			} catch (Exception e)
			{
				String err = "The server was shut down. Try again later.";
				userGUI.append(err);
				userGUI.resetGUI();
				break;
			}

			int type = packet.getType();
			if (type == Packet.MESSAGE || type == Packet.CONTROL_MSG)
			{
				String msg = packet.getData();
				userGUI.append(msg);
			} else if (type == Packet.FAILED || type == Packet.CLIENT_LEFT)
			{
				String msg = packet.getData();
				userGUI.append(msg);
				userGUI.resetGUI();
				break;
			}
		}
	}

}
