package client;

import general.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Date;

import javax.swing.*;

public class ClientGUI extends JFrame implements ActionListener, UserGUI
{
	private static final long serialVersionUID = 1L;
	private JTextField serverText, portText, usernameText;
	private JTextArea log, writeText;
	private JButton login, logout, send;
	private boolean connected;
	private Client client;

	public ClientGUI()
	{
		super("Chat Client");

		getContentPane().setLayout(null);
		JLabel lblServer = new JLabel("Server:");
		lblServer.setBounds(13, 14, 46, 14);
		getContentPane().add(lblServer);

		serverText = new JTextField("localhost");
		serverText.setBounds(69, 11, 86, 20);
		getContentPane().add(serverText);
		serverText.setColumns(10);

		JLabel lblPortNumber = new JLabel("Port Number:");
		lblPortNumber.setBounds(165, 14, 84, 14);
		getContentPane().add(lblPortNumber);

		portText = new JTextField("2500");
		portText.setBounds(259, 11, 86, 20);
		getContentPane().add(portText);
		portText.setColumns(10);

		JLabel lblUsername = new JLabel("Username:");
		lblUsername.setBounds(13, 45, 63, 14);
		getContentPane().add(lblUsername);

		usernameText = new JTextField();
		usernameText.setBounds(94, 42, 86, 20);
		getContentPane().add(usernameText);
		usernameText.setColumns(10);

		login = new JButton("Login");
		login.addActionListener(this);
		login.setBounds(190, 41, 79, 23);
		getContentPane().add(login);

		logout = new JButton("Logout");
		logout.setEnabled(false);
		logout.addActionListener(this);
		logout.setBounds(279, 41, 79, 23);
		getContentPane().add(logout);

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(13, 70, 351, 365);
		getContentPane().add(scrollPane);

		log = new JTextArea();
		log.setLineWrap(true);
		log.setEditable(false);
		scrollPane.setViewportView(log);

		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(13, 446, 278, 109);
		getContentPane().add(scrollPane_1);

		writeText = new JTextArea();
		writeText.setLineWrap(true);
		scrollPane_1.setViewportView(writeText);
		writeText.addKeyListener(new KeyListener()
		{

			@Override
			public void keyTyped(KeyEvent arg0)
			{
				if (arg0.getKeyChar() == KeyEvent.VK_ENTER && connected)
				{
					client.writePacket(new Packet(Packet.MESSAGE, writeText.getText(), new Date()));
					writeText.setText("");
				}
			}

			@Override
			public void keyReleased(KeyEvent arg0)
			{
			}

			@Override
			public void keyPressed(KeyEvent arg0)
			{
			}
		});

		send = new JButton("Send");
		send.setEnabled(false);
		send.addActionListener(this);
		send.setBounds(301, 446, 63, 109);
		getContentPane().add(send);

		setSize(390, 605);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public static void main(String[] args)
	{
		new ClientGUI();
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		Object o = e.getSource();
		if (o == logout)
		{
			client.writePacket(new Packet(Packet.LOGOUT, 0, 0, null, new Date()));
			return;
		} else if (o == send && connected)
		{
			client.writePacket(new Packet(Packet.MESSAGE, writeText.getText(), new Date()));
			writeText.setText("");
			return;
		} else if (o == login)
		{
			String username = usernameText.getText().trim();
			if (username.length() == 0)
			{
				append("A username is required to log in.");
				return;
			}

			String server = serverText.getText().trim();
			if (server.length() == 0)
			{
				append("A server is required to log in.");
				return;
			}
			String portNumber = portText.getText().trim();
			int port;
			try
			{
				port = Integer.parseInt(portNumber);
			} catch (Exception en)
			{
				append("Invalid port number.");
				return;
			}

			client = new Client(server, port, username, this);

			if (!client.connect())
			{
				append("Connection Failed.");
				return;
			}

			login.setEnabled(false);
			logout.setEnabled(true);
			send.setEnabled(true);
			serverText.setEditable(false);
			portText.setEditable(false);
			usernameText.setEditable(false);
			connected = true;
			setTitle(username);
		}
	}

	public void append(String str)
	{
		log.append(str + "\n");
		log.setCaretPosition(log.getText().length() - 1);
	}

	public void resetGUI()
	{
		login.setEnabled(true);
		logout.setEnabled(false);
		send.setEnabled(false);
		serverText.setEditable(true);
		portText.setEditable(true);
		usernameText.setEditable(true);
		connected = false;
	}
}
