package general;

import java.io.*;
import java.net.*;
import java.security.*;
import javax.net.ssl.*;

public class User
{
	protected String server;
	protected int port;
	protected ObjectInputStream inputStream;
	protected ObjectOutputStream outputStream;
	protected UserGUI userGUI;

	protected SSLSocket s;
	private KeyStore cks;
	private KeyStore sks;
	private SSLContext sc;
	static private final String pp = "clientpw";
	static private SecureRandom sR;

	public User(String server, int port, UserGUI userGUI)
	{
		this.server = server;
		this.port = port;
		this.userGUI = userGUI;
	}

	private void scks() throws GeneralSecurityException, IOException
	{
		cks = KeyStore.getInstance("JKS");
		cks.load(new FileInputStream("client.private"), pp.toCharArray());
	}

	public boolean connect()
	{
		try
		{
			ssk();
			//scks();
			ssc();

			SSLSocketFactory sf = sc.getSocketFactory();
			s = (SSLSocket) sf.createSocket(InetAddress.getByName(server).getHostAddress(), port);

			inputStream = new ObjectInputStream(s.getInputStream());
			outputStream = new ObjectOutputStream(s.getOutputStream());
		} catch (IOException e1)
		{
			String err = "Exception creating new Input/output Streams: " + e1;
			userGUI.append(err);
			userGUI.resetGUI();
			return false;
		} catch (Exception e1)
		{
			String err = "Error connectiong to server:" + e1;
			userGUI.append(err);
			userGUI.resetGUI();
			return false;
		}

		String err = "Connection accepted " + s.getInetAddress() + ":" + s.getPort();
		userGUI.append(err);
		return true;
	}
	
	private void ssk() throws GeneralSecurityException, IOException
	{
		sks = KeyStore.getInstance("JKS");
		sks.load(new FileInputStream("server.public"), "public".toCharArray());
	}
	
	public void writePacket(Packet packet)
	{
		try
		{
			outputStream.writeObject(packet);
		} catch (IOException e)
		{
			String err = "Exception writing to server: " + e;
			userGUI.append(err);
		}
	}

	private void ssc() throws GeneralSecurityException, IOException
	{
		TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
		tmf.init(sks);

		KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
		kmf.init(cks, pp.toCharArray());

		sc = SSLContext.getInstance("TLS");
		sc.init(kmf.getKeyManagers(), tmf.getTrustManagers(), sR);
	}
	
	public void close()
	{
		// inputStream.close();
		try
		{
			if (inputStream != null)
				inputStream.close();
			if (outputStream != null)
				outputStream.close();
			if (s != null)
				s.close();
		} catch (Exception e)
		{
		}
		userGUI.resetGUI();
	}
}
