package helpDesk;

import general.*;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;


public class HelpDesk extends User implements Runnable
{
	private String username;
	private String password;
	HashMap<String, String> myUsers;

	public HelpDesk(String server, int port, String username, String password, HelpDeskGUI userGUI)
	{
		super(server, port, userGUI);
		this.username = username;
		this.password = password;
		this.myUsers = new HashMap<String, String>();
	}

	@Override
	public boolean connect()
	{
		if (super.connect())
		{
			new Thread(this).start();
			try
			{
				outputStream.writeObject(username + "&" + password);
			} catch (IOException e)
			{
				String err = "Exception on logging in : " + e;
				userGUI.append(err);
				close();
				return false;
			}
		}
		return true;
	}

	@Override
	public void run()
	{
		while (true)
		{
			Packet packet;
			try
			{
				packet = (Packet) inputStream.readObject();
				if (new Date().getTime() - packet.getTimeStamp().getTime() >= 60 * 1000)
					continue;
			} catch (Exception e)
			{
				String err = "The server was shut down. Try again later.";
				userGUI.append(err);
				userGUI.resetGUI();
				break;
			}

			int type = packet.getType();

			if (type == Packet.MESSAGE)
			{
				String msg = packet.getData();
				int dest = packet.getDestination();
				((HelpDeskGUI) userGUI).append(msg, dest);
			} else if (type == Packet.FAILED)
			{
				String msg = packet.getData();
				userGUI.append(msg);
				userGUI.resetGUI();
				break;
			} else if (type == Packet.CONTROL_MSG)
			{
				String msg = packet.getData();
				userGUI.append(msg);
			} else if (type == Packet.CLIENT_LEFT)
			{
				((HelpDeskGUI) userGUI).removeUser(packet.getData(), false);
			} else if (type == Packet.NEW_CLIENT)
			{
				((HelpDeskGUI) userGUI).addUser(packet.getData(), false);
			} else if (type == Packet.NEW_HD)
			{
				((HelpDeskGUI) userGUI).addUser(packet.getData(), true);
			} else if (type == Packet.HD_LEFT)
			{
				((HelpDeskGUI) userGUI).removeUser(packet.getData(), true);
			} else if (type == Packet.CONNECT)
			{
				((HelpDeskGUI) userGUI).addTab(packet.getData());
			} else if (type == Packet.DISCONNECT)
			{
				((HelpDeskGUI) userGUI).removeTab(packet.getData());
			}
		}
	}
}
