package helpDesk;

import general.*;

import javax.swing.*;
import java.awt.event.*;
import java.util.Date;

public class HelpDeskGUI extends JFrame implements ActionListener, UserGUI
{
	private static final long serialVersionUID = 1L;
	private JTextField serverText, portText, usernameText;
	private JPasswordField passwordText;
	private JTextArea log;
	private JTabbedPane tabbedPane;
	private JButton login, logout;
	private JLabel lblPassword;
	private HelpDesk helpDesk;
	private JScrollPane scrollList;
	private DefaultListModel listModel;
	private JList list;
	private JComboBox hdBox, clientBox;
	private JButton btnChat, transfer;

	public HelpDeskGUI()
	{
		super("Chat Help-Desk");

		getContentPane().setLayout(null);
		JLabel lblServer = new JLabel("Server:");
		lblServer.setBounds(13, 14, 46, 14);
		getContentPane().add(lblServer);

		serverText = new JTextField("localhost");
		serverText.setBounds(69, 11, 86, 20);
		getContentPane().add(serverText);
		serverText.setColumns(10);

		JLabel lblPortNumber = new JLabel("Port Number:");
		lblPortNumber.setBounds(165, 14, 84, 14);
		getContentPane().add(lblPortNumber);

		portText = new JTextField("2500");
		portText.setBounds(259, 11, 86, 20);
		getContentPane().add(portText);
		portText.setColumns(10);

		JLabel lblName = new JLabel("Username:");
		lblName.setBounds(13, 45, 63, 14);
		getContentPane().add(lblName);

		usernameText = new JTextField();
		usernameText.setBounds(86, 42, 109, 20);
		getContentPane().add(usernameText);
		usernameText.setColumns(10);

		lblPassword = new JLabel("Password:");
		lblPassword.setBounds(205, 45, 63, 14);
		getContentPane().add(lblPassword);

		passwordText = new JPasswordField();
		passwordText.setBounds(269, 42, 109, 20);
		getContentPane().add(passwordText);
		passwordText.setColumns(10);

		login = new JButton("Login");
		login.setBounds(408, 41, 107, 23);
		login.addActionListener(this);
		getContentPane().add(login);

		logout = new JButton("Logout");
		logout.setBounds(408, 10, 107, 23);
		logout.setEnabled(false);
		logout.addActionListener(this);
		getContentPane().add(logout);

		JLabel lblHelpdeskUsers = new JLabel("Help-Desk Users:");
		lblHelpdeskUsers.setBounds(408, 75, 100, 14);
		getContentPane().add(lblHelpdeskUsers);

		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		tabbedPane.setBounds(13, 149, 385, 442);
		getContentPane().add(tabbedPane);

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(13, 82, 385, 56);
		getContentPane().add(scrollPane);

		log = new JTextArea();
		scrollPane.setViewportView(log);
		log.setEditable(false);
		log.setLineWrap(true);

		scrollList = new JScrollPane();
		scrollList.setBounds(408, 101, 107, 318);
		getContentPane().add(scrollList);
		listModel = new DefaultListModel();
		list = new JList(listModel);
		scrollList.setViewportView(list);
		list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		list.setLayoutOrientation(JList.VERTICAL);
		list.setVisibleRowCount(-1);

		btnChat = new JButton("Start Chat");
		btnChat.setBounds(408, 430, 107, 25);
		btnChat.setEnabled(false);
		btnChat.addActionListener(this);
		getContentPane().add(btnChat);

		JLabel lblClientName = new JLabel("Client Name:");
		lblClientName.setBounds(408, 477, 107, 14);
		getContentPane().add(lblClientName);

		clientBox = new JComboBox();
		clientBox.setBounds(408, 493, 107, 20);
		getContentPane().add(clientBox);

		JLabel lblHelpdeskName = new JLabel("Help-Desk Name:");
		lblHelpdeskName.setBounds(408, 522, 107, 14);
		getContentPane().add(lblHelpdeskName);

		hdBox = new JComboBox();
		hdBox.setBounds(408, 537, 107, 20);
		getContentPane().add(hdBox);

		transfer = new JButton("Transfer");
		transfer.addActionListener(this);
		transfer.setEnabled(false);
		transfer.setBounds(408, 568, 107, 23);
		getContentPane().add(transfer);

		setSize(545, 649);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public static void main(String[] args)
	{
		new HelpDeskGUI();
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		Object o = e.getSource();

		if (o == logout)
		{
			helpDesk.writePacket(new Packet(Packet.LOGOUT, 0, 0, null, new Date()));
			return;
		} else if (o == btnChat)
		{
			String hd = (String) list.getSelectedValue();
			if (hd == null)
			{
				append("Selecting a user is required to start a chat.");
				return;
			}
			for (int i = 0; i < tabbedPane.getTabCount(); i++)
			{
				if (tabbedPane.getTitleAt(i).equals(hd))
				{
					append("A chat has already started with this user.");
					return;
				}
			}
			helpDesk.writePacket(new Packet(Packet.CONNECT, helpDesk.myUsers.get(hd), new Date()));
		} else if (o == login)
		{
			String username = usernameText.getText().trim();
			if (username.length() == 0)
			{
				append("A username is required to log in.");
				return;
			}

			String password = passwordText.getText().trim();
			if (password.length() == 0)
			{
				append("A password is required to log in.");
				return;
			}

			String server = serverText.getText().trim();
			if (server.length() == 0)
			{
				append("A server is required to log in.");
				return;
			}

			String portNumber = portText.getText().trim();
			int port;
			try
			{
				port = Integer.parseInt(portNumber);
			} catch (Exception en)
			{
				append("Invalid port number.");
				return;
			}

			helpDesk = new HelpDesk(server, port, username, password, this);

			if (!helpDesk.connect())
			{
				append("Connection Failed.");
				return;
			}

			login.setEnabled(false);
			logout.setEnabled(true);
			serverText.setEditable(false);
			portText.setEditable(false);
			usernameText.setEditable(false);
			passwordText.setEditable(false);
			setTitle(username);
		} else if (o == transfer)
		{
			String client = (String) clientBox.getSelectedItem();
			if (client == null)
				return;

			String hd = (String) hdBox.getSelectedItem();
			if (hd == null)
				return;

			helpDesk.writePacket(new Packet(Packet.TRANSFER, helpDesk.myUsers.get(hd) + "&" + helpDesk.myUsers.get(client),
					new Date()));
			removeUser(client + "&" + helpDesk.myUsers.get(client), false);
		}
	}

	public void append(String str)
	{
		log.append(str + "\n");
		log.setCaretPosition(log.getText().length() - 1);
	}

	public void append(String str, int destination)
	{
		for (int i = 0; i < tabbedPane.getComponentCount(); i++)
		{
			JPanel panel = (JPanel) tabbedPane.getComponentAt(i);
			if (panel.getName().equals(destination + ""))
			{
				JTextArea chat = (JTextArea) ((JScrollPane) panel.getComponent(0)).getViewport().getView();
				chat.append(str + "\n");
				chat.setCaretPosition(chat.getText().length() - 1);
				break;
			}
		}
	}

	public void resetGUI()
	{
		login.setEnabled(true);
		logout.setEnabled(false);
		btnChat.setEnabled(false);
		serverText.setEditable(true);
		portText.setEditable(true);
		usernameText.setEditable(true);
		passwordText.setEditable(true);
		listModel.clear();
		tabbedPane.removeAll();
		hdBox.removeAllItems();
		clientBox.removeAllItems();
		transfer.setEnabled(false);
	}

	public void addTab(String data)
	{
		int d = data.indexOf("&");
		String username = data.substring(0, d);
		final String id = data.substring(d + 1);

		final JPanel panel = new JPanel();
		panel.setName(id);
		panel.setLayout(null);

		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(2, 1, 373, 290);
		panel.add(scrollPane);

		final JTextArea chat = new JTextArea();
		scrollPane.setViewportView(chat);
		chat.setLineWrap(true);
		chat.setEditable(false);

		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(2, 301, 274, 109);
		panel.add(scrollPane_1);

		final JTextArea writeText = new JTextArea();
		scrollPane_1.setViewportView(writeText);
		writeText.setLineWrap(true);

		final JButton btnSend = new JButton("Send");
		btnSend.setBounds(284, 301, 88, 50);
		btnSend.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				helpDesk.writePacket(new Packet(Packet.MESSAGE, 0, Integer.parseInt(id), writeText.getText(), new Date()));
				writeText.setText("");
				return;
			}
		});

		writeText.addKeyListener(new KeyListener()
		{

			@Override
			public void keyTyped(KeyEvent arg0)
			{
				if (arg0.getKeyChar() == KeyEvent.VK_ENTER)
				{
					helpDesk.writePacket(new Packet(Packet.MESSAGE, 0, Integer.parseInt(id), writeText.getText(), new Date()));
					writeText.setText("");
				}
			}

			@Override
			public void keyReleased(KeyEvent arg0)
			{
			}

			@Override
			public void keyPressed(KeyEvent arg0)
			{
			}
		});
		panel.add(btnSend);

		final JButton btnClose = new JButton("Close");
		btnClose.setBounds(284, 356, 88, 50);
		btnClose.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				helpDesk.writePacket(new Packet(Packet.DISCONNECT, id, new Date()));
				return;
			}
		});
		panel.add(btnClose);

		tabbedPane.addTab(username, null, panel, null);
	}

	public void removeTab(String data)
	{
		int d = data.indexOf("&");
		String id = data.substring(d + 1);

		for (int i = 0; i < tabbedPane.getComponentCount(); i++)
		{
			JPanel panel = (JPanel) tabbedPane.getComponentAt(i);
			if (panel.getName().equals(id))
			{
				tabbedPane.remove(panel);
				break;
			}
		}
	}

	public void addUser(String data, boolean isHD)
	{
		int d = data.indexOf("&");
		String username = data.substring(0, d);
		String id = data.substring(d + 1);

		helpDesk.myUsers.put(username, id);

		if (isHD)
		{
			listModel.addElement(username);
			hdBox.addItem(username);
			btnChat.setEnabled(true);
		} else
		{
			clientBox.addItem(username);
			addTab(data);
		}
		transfer.setEnabled(true);
	}

	public void removeUser(String data, boolean isHD)
	{
		int d = data.indexOf("&");
		String username = data.substring(0, d);

		helpDesk.myUsers.remove(username);

		if (isHD)
		{
			removeTab(data);
			listModel.removeElement(username);
			hdBox.removeItem(username);
			if (listModel.size() < 1)
				btnChat.setEnabled(false);
		} else
		{
			clientBox.removeItem(username);
			removeTab(data);
		}
	}
}
