package server;

import general.*;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.Date;


public class ClientProcessor extends ConnectionProcessor implements Runnable
{
	public int destination;

	public ClientProcessor(Server server, Socket socket, ObjectInputStream inputStream, ObjectOutputStream outputStream,
			String username)
	{
		super(server, socket, inputStream, outputStream, username);

		destination = server.asign(username, id);
		writePacket(new Packet(Packet.CONTROL_MSG, "You are successfully loged in.", new Date()));
		server.print("'" + username + "' is now connected.");
	}

	public void logout()
	{
		ConnectionProcessor hd = server.processers.get(destination);
		hd.writePacket(new Packet(Packet.CLIENT_LEFT, username + "&" + id, new Date()));

		server.re(id);
		close();
	}

	@Override
	public void run()
	{
		boolean logedIn = true;
		while (logedIn && server.onOff)
		{
			Packet packet;
			try
			{
				packet = (Packet) inputStream.readObject();
				if (new Date().getTime() - packet.getTimeStamp().getTime() >= 60 * 1000)
					continue;
			} catch (IOException e)
			{
				if (server.onOff)
				{
					server.print("'" + username + "' can't read streams: " + e);
					logout();
				}
				break;
			} catch (ClassNotFoundException e2)
			{
				break;
			}

			int type = packet.getType();

			if (type == Packet.MESSAGE)
			{
				server.forw(type, id, destination, username + ": " + packet.getData());
			} else if (type == Packet.LOGOUT)
			{
				logout();
				logedIn = false;
			}
		}
	}

}
