package server;

import general.*;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;


public abstract class ConnectionProcessor
{
	public Server server;
	public Socket socket;
	public ObjectInputStream inputStream;
	public ObjectOutputStream outputStream;
	public int id;
	public String username;

	public ConnectionProcessor(Server server, Socket socket, ObjectInputStream inputStream, ObjectOutputStream outputStream,
			String username)
	{
		id = server.globalID++;
		this.server = server;
		this.socket = socket;
		this.inputStream = inputStream;
		this.outputStream = outputStream;
		this.username = username;
	}

	public void close()
	{
		try
		{
			if (outputStream != null)
				outputStream.close();
			if (inputStream != null)
				inputStream.close();
			if (socket != null)
				socket.close();
		} catch (Exception e)
		{
			server.print("'" + username + "' can't close streams: " + e);
		}
	}

	public boolean writePacket(Packet packet)
	{
		if (!socket.isConnected() || !server.onOff)
		{
			close();
			return false;
		}

		try
		{
			outputStream.writeObject(packet);
		} catch (Exception e)
		{
			server.print("Server can't write to '" + username + "': " + e);
		}
		return true;
	}
}
