package server;

import general.*;
import java.io.*;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Date;
import java.util.*;

public class HelpDeskProcessor extends ConnectionProcessor implements Runnable
{
	public FileWriter log;

	public HelpDeskProcessor(Server server, Socket socket, ObjectInputStream inputStream, ObjectOutputStream outputStream,
			String username)
	{
		super(server, socket, inputStream, outputStream, username);

		server.seG.listModel.addElement(username);
		for (HelpDeskProcessor hd : server.hps)
		{
			hd.writePacket(new Packet(Packet.NEW_HD, username + "&" + id, new Date()));
			writePacket(new Packet(Packet.NEW_HD, hd.username + "&" + hd.id, new Date()));
		}
		server.hps.addFirst(this);

		try
		{
			log = new FileWriter("logs/" + username + ".txt", true);
		} catch (Exception e)
		{
			server.print("Server could not create log file for '" + username + "'");
		}

		writePacket(new Packet(Packet.CONTROL_MSG, "You are successfully loged in.", new Date()));
		server.print("'" + username + "' is now connected.");
	}

	@Override
	public boolean writePacket(Packet packet)
	{
		record(packet);
		return super.writePacket(packet);
	}

	private void logout()
	{
		for (int i : server.processers.keySet())
		{
			if (server.processers.get(i) instanceof ClientProcessor)
			{
				if (((ClientProcessor) server.processers.get(i)).destination == id)
				{
					server.processers.get(i).writePacket(
							new Packet(Packet.FAILED, "The Help-Desk has left. Try again later.", new Date()));
				}
			}
		}
		
		server.seG.listModel.removeElement(username);

		for (HelpDeskProcessor hd : server.hps)
		{
			hd.writePacket(new Packet(Packet.HD_LEFT, username + "&" + id, new Date()));
		}
		server.re(id);
		close();
	}

	private void record(Packet packet)
	{
		try
		{
			log = new FileWriter("logs/" + username + ".txt", true);
			String info = packet.getTimeStamp() + ": ";
			info += packet.getType() + ": ";
			info += packet.getSource() + ": ";
			info += packet.getDestination() + ": ";
			info += packet.getData() + System.getProperty("line.separator");
			log.write(info);
			log.close();
		} catch (IOException e)
		{
			server.print("Server could not record logs: " + e);
			e.printStackTrace();
		}
	}

	@Override
	public void run()
	{
		boolean logedIn = true;
		while (logedIn && server.onOff)
		{
			Packet packet;
			try
			{
				packet = (Packet) inputStream.readObject();
				record(packet);
				if (new Date().getTime() - packet.getTimeStamp().getTime() >= 60 * 1000)
					continue;
			} catch (IOException e)
			{
				if (server.onOff)
				{
					server.print("'" + username + "' can't read streams: " + e);
					logout();
				}
				break;
			} catch (ClassNotFoundException e2)
			{
				break;
			}

			int type = packet.getType();

			if (type == Packet.MESSAGE)
			{
				server.forw(packet.getType(), id, packet.getDestination(), username + ": " + packet.getData());
			} else if (type == Packet.CONNECT)
			{
				String data = packet.getData();
				HelpDeskProcessor helpDesk = (HelpDeskProcessor) server.processers.get(Integer.parseInt(data));
				helpDesk.writePacket(new Packet(Packet.CONNECT, username + "&" + id, new Date()));
				writePacket(new Packet(Packet.CONNECT, helpDesk.username + "&" + helpDesk.id, new Date()));
			} else if (type == Packet.DISCONNECT)
			{
				String data = packet.getData();
				ConnectionProcessor user = server.processers.get(Integer.parseInt(data));
				if (user instanceof HelpDeskProcessor)
					user.writePacket(new Packet(Packet.DISCONNECT, username + "&" + id, new Date()));
				else
				{
					user.writePacket(new Packet(Packet.FAILED, "The Help-Desk has left. Try again later.", new Date()));
					((ClientProcessor)user).logout();
				}
				writePacket(new Packet(Packet.DISCONNECT, user.username + "&" + user.id, new Date()));

			} else if (type == Packet.TRANSFER)
			{
				String data = packet.getData();
				int d = data.indexOf("&");
				String hdId = data.substring(0, d);
				String clientId = data.substring(d + 1);
				
				ClientProcessor client = (ClientProcessor) server.processers.get(Integer.parseInt(clientId));
				HelpDeskProcessor hd = (HelpDeskProcessor) server.processers.get(Integer.parseInt(hdId));

				hd.writePacket(new Packet(Packet.NEW_CLIENT, client.username + "&" + client.id, new Date()));
				server.hps.remove(hd);
				server.hps.addLast(hd);
				client.destination = hd.id;
			} else if (type == Packet.LOGOUT)
			{
				logout();
				logedIn = false;
			}
		}
	}
}
