package server;

import general.Packet;

import java.io.*;
import java.net.*;
import java.security.*;
import java.security.spec.KeySpec;
import java.util.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.net.ssl.*;
import java.text.SimpleDateFormat;


public class Server implements Runnable
{
	public int globalID;
	public HashMap<Integer, ConnectionProcessor> processers;
	public HashMap<InetAddress, Integer> ia;
	public LinkedList<HelpDeskProcessor> hps;
	public ServerGUI seG;
	public SimpleDateFormat tf;
	public int port;
	public boolean onOff;
	public HashMap<String, PasswordInfo> ps;

	public SSLServerSocket ss;
	private KeyStore cks;
	private KeyStore sks;
	private SSLContext sc;
	static private final String pp = "serverpw";
	static private SecureRandom sr;

	public Server(int port, ServerGUI serverGUI)
	{
		this.globalID = 0;
		this.port = port;
		this.seG = serverGUI;
		this.tf = new SimpleDateFormat("HH:mm:ss");
		this.processers = new HashMap<Integer, ConnectionProcessor>();
		this.ia = new HashMap<InetAddress, Integer>();
		this.hps = new LinkedList<HelpDeskProcessor>();
		this.ps = new HashMap<String, PasswordInfo>();
	}

	public void process(Socket socket)
	{
		try
		{
			ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
			ObjectInputStream in = new ObjectInputStream(socket.getInputStream());

			String da = (String) in.readObject();
			int d = da.indexOf("&");
			if (d >= 1)
				processe(socket, out, in, da);
			else
				processa(socket, out, in, da);

		} catch (Exception e)
		{
			print("Server could not create I/O Streams: " + e);
		}
	}

	public void processe(Socket socket, ObjectOutputStream out, ObjectInputStream in, String data)
	{
		int d = data.indexOf("&");
		String username = data.substring(0, d);
		String password = data.substring(d + 1);
		try
		{
			if (!doit(username, password, out))
			{
				socket.close();
				out.close();
				in.close();
				return;
			}
		} catch (Exception e)
		{
			print("Server could not process a help-desk.: " + e);
			return;
		}
		HelpDeskProcessor user = new HelpDeskProcessor(this, socket, in, out, username);
		processers.put(user.id, user);
		new Thread(user).start();
	}

	public void processa(Socket socket, ObjectOutputStream out, ObjectInputStream in, String da)
	{
		String username = da;
		boolean toReturn = true;
		String msg = "";

		for (ConnectionProcessor user : processers.values())
		{
			if (user.username.equals(username))
			{
				msg = "Username is taken. Try another one!";
				toReturn = false;
			}
		}

		if (hps.size() < 1)
		{
			msg = "There are no Help-Desk users right now. Try again later.";
			toReturn = false;
		}

		if (pda(socket.getInetAddress()))
		{
			msg = "Too many connections from the same IP address. Try again later.";
			toReturn = false;
		}

		if (!toReturn)
		{
			try
			{
				out.writeObject(new Packet(Packet.FAILED, msg, new Date()));
				socket.close();
				out.close();
				in.close();
			} catch (Exception e)
			{
				print("Server could not end processing a new client: " + e);
			}
		} else
		{
			ClientProcessor user = new ClientProcessor(this, socket, in, out, username);
			processers.put(user.id, user);
			new Thread(user).start();
		}
	}

	protected void done() throws Exception
	{
		onOff = false;
		ss.close();
		for (int id : processers.keySet())
		{
			processers.get(id).writePacket(new Packet(Packet.FAILED, "Server was shut down. Try again later.", new Date()));
			processers.get(id).close();
		}
		processers.clear();
		globalID = 0;
	}

	public void forw(int type, int source, int destination, String msg)
	{
		String time = tf.format(new Date());
		String message = time + ":  " + msg;

		Packet packetSrc = new Packet(type, source, destination, message, new Date());
		Packet packetDest = new Packet(type, destination, source, message, new Date());

		processers.get(destination).writePacket(packetDest);
		processers.get(source).writePacket(packetSrc);
	}
	
	private void ssks() throws GeneralSecurityException, IOException
	{
		sks = KeyStore.getInstance("JKS");
		sks.load(new FileInputStream("server.private"), pp.toCharArray());
	}
	
	public synchronized int asign(String username, int id)
	{
		HelpDeskProcessor helpDesk = hps.removeFirst();
		hps.addLast(helpDesk);
		helpDesk.writePacket(new Packet(Packet.NEW_CLIENT, username + "&" + id, new Date()));
		return helpDesk.id;
	}

	public synchronized void re(int id)
	{
		if (processers.containsKey(id))
		{
			ConnectionProcessor processor = processers.get(id);
			print("'" + processor.username + "' has left.");
			processers.remove(id);
			if (processor instanceof HelpDeskProcessor)
			{
				hps.remove(processor);
			}
		}
	}

	public void print(String message)
	{
		String msg = tf.format(new Date()) + " " + message;
		seG.append(msg);
	}
	
	private void scks() throws Exception
	{
		cks = KeyStore.getInstance("JKS");
		cks.load(new FileInputStream("client.public"), "public".toCharArray());
	}
	
	public String gep(String p, String s) throws Exception
	{
		String algorithm = "PBKDF2WithHmacSHA1";
		int dkl = 160;
		int i = 20000;
		KeySpec sc = new PBEKeySpec(p.toCharArray(), htb(s), i, dkl);
		SecretKeyFactory f = SecretKeyFactory.getInstance(algorithm);
		return bth(f.generateSecret(sc).getEncoded());
	}
	
	private void ssc() throws Exception
	{
		TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
		tmf.init(cks);
		KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
		kmf.init(sks, pp.toCharArray());
		sc = SSLContext.getInstance("TLS");
		sc.init(kmf.getKeyManagers(), tmf.getTrustManagers(), sr);
	}
	
	public boolean a(String ap, String ep, String s) throws Exception
	{
		String eap = gep(ap, s);
		return eap.equals(ep);
	}
	
	private boolean doit(String username, String password, ObjectOutputStream out) throws Exception
	{
		PasswordInfo pass;
		if (ps.containsKey(username))
			pass = ps.get(username);
		else
		{
			pass = new PasswordInfo(username);
			if (pass.wasFound())
				ps.put(username, pass);
		}

		boolean toReturn = true;
		String msg = "";

		if (!pass.wasFound() || !a(password, pass.getHex(), pass.getSalt()))
		{
			pass.failedAttempet();
			msg = "Wrong Username or Password!";
			toReturn = false;
		}

		if (pass.isLocked())
		{
			msg = "Too many password attempts. Try after one hour!";
			toReturn = false;
		}

		for (HelpDeskProcessor hd : hps)
		{
			if (hd.username.equals(username))
			{
				msg = "User is already logged in!";
				toReturn = false;
			}
		}

		if (!toReturn)
		{
			out.writeObject(new Packet(Packet.FAILED, msg, new Date()));
		}

		return toReturn;
	}
	

	public String bth(byte[] a)
	{
		StringBuilder sb = new StringBuilder();
		for (byte b : a)
			sb.append(String.format("%02x", b & 0xff));
		return sb.toString();
	}

	private boolean pda(InetAddress inetAddress)
	{
		if (ia.containsKey(inetAddress))
		{
			int count = ia.get(inetAddress);
			if (count <= 20)
			{
				ia.remove(inetAddress);
				ia.put(inetAddress, ++count);
				return false;
			} else
				return true;
		} else
		{
			ia.put(inetAddress, 1);
			return false;
		}
	}

	@Override
	public void run()
	{
		onOff = true;
		try
		{
			//scks();
			ssks();
			ssc();

			SSLServerSocketFactory sf = sc.getServerSocketFactory();
			ss = (SSLServerSocket) sf.createServerSocket(port);
			ss.setNeedClientAuth(false);

			print("Server is running on port " + port + ".");
			while (onOff)
			{
				Socket socket = ss.accept();

				if (onOff)
				{
					process(socket);
				}
			}
		} catch (Exception e)
		{
			String msg = tf.format(new Date()) + " Exception on starting the server: " + e;
			print(msg);
		}
		seG.resetGUI();
	}
	
	public byte[] htb(String string)
	{
		int len = string.length();
		byte[] data = new byte[len / 2];
		for (int i = 0; i < len; i += 2)
			data[i / 2] = (byte) ((Character.digit(string.charAt(i), 16) << 4) + Character.digit(string.charAt(i + 1), 16));
		return data;
	}
}